import argparse
import json

from todoist import TodoistAPI
import gspread
from gspread.exceptions import WorksheetNotFound
from oauth2client.service_account import ServiceAccountCredentials


class BaseCommand(object):
    help_text = ''

    def __init__(self, subcommand_creator):

        # keep a local copy of the config file which is useful during
        # autocompletion
        self.config = None

        # set up the subcommand options
        self.subcommand_creator = subcommand_creator
        self.option_parser = self.subcommand_creator.add_parser(
            self.get_command_name(),
            help=self.help_text,
            description=self.help_text,
        )
        self.add_command_line_options()

    def get_command_name(self):
        """The command name defaults to the name of the module."""
        return self.__module__.rsplit('.', 1)[1]

    def add_command_line_options(self):
        self.option_parser.add_argument(
            '--todoist',
            type=argparse.FileType('r'),
            metavar='JSONFILE',
            default='todoist.json',
            help='todoist credentials file in json format',
        )
        self.option_parser.add_argument(
            '--google',
            type=argparse.FileType('r'),
            metavar='JSONFILE',
            default='google.json',
            help='google credentials file in json format',
        )
        self.option_parser.add_argument(
            '--debug',
            action='store_true',
            help='log output on command line, NOT google spreadsheet'
        )

    def execute(self, todoist=None, google=None, debug=None, **kwargs):
        """Common execution workflows are handled here"""

        # create an authenticated instance of the TodoistAPI
        credentials = json.load(todoist)
        self.todoist_api = TodoistAPI(**credentials)

        # authenticate to google
        google_keys = json.load(google)
        credentials = ServiceAccountCredentials.from_json_keyfile_name(
            google.name,
            ['https://spreadsheets.google.com/feeds'],
        )
        gdrive = gspread.authorize(credentials)
        self.gdrive_workbook = gdrive.open_by_url(google_keys['workbook_url'])

    def get_or_create_worksheet(self, title, header):
        try:
            worksheet = self.gdrive_workbook.worksheet(title)
        except WorksheetNotFound:
            worksheet = self.gdrive_workbook.add_worksheet(title, 1, 26)
            worksheet.insert_row(header)
        return worksheet



import arrow

TODOIST_FORMAT = 'ddd DD MMM YYYY HH:mm:ss ZZ'


def get_overdue_items(todoist_api):
    """get overdue items from authenticated todoist_api instance"""
    overdue_items = []
    today = arrow.utcnow().to('local').date()
    todoist_api.items.sync()
    for item in todoist_api.items.all():
        if item['due_date_utc']:
            due_datetime = arrow.get(item['due_date_utc'], [TODOIST_FORMAT])
            local_due_datetime = due_datetime.to('local')
            local_due_date = local_due_datetime.date()
            if local_due_date <= today:
                overdue_items.append(item)
    return overdue_items



import argparse

from ..base import BaseCommand
from ..utils import get_command_line_parser
from . import n
from . import time
from . import priority

COMMAND_MODULES = (n, time, priority, )


class Command(BaseCommand):
    help_text = (
        "Track statistics on overdue tasks"
    )

    def add_command_line_options(self):
        super(Command, self).add_command_line_options()

        # add subcommands using the same pattern
        self.option_parser = get_command_line_parser(
            command_line_parser=self.option_parser,
            command_modules=COMMAND_MODULES,
        )

    def execute(self, **kwargs):
        raise NotImplementedError("this should never happen")



import datetime

from ..base import BaseCommand
from ...overdue import get_overdue_items


class Command(BaseCommand):
    help_text = (
        "Count the number of overdue tasks"
    )

    def add_command_line_options(self):
        super(Command, self).add_command_line_options()

    def execute(self, **kwargs):
        super(Command, self).execute(**kwargs)

        overdue_items = get_overdue_items(self.todoist_api)

        if kwargs['debug']:
            print '%d overdue items' % len(overdue_items)
        else:
            worksheet = self.get_or_create_worksheet(
                'overdue n',
                ['date', 'n'],
            )
            worksheet.insert_row([
                datetime.datetime.now().date(),
                len(overdue_items),
            ], 2)



import datetime

from pytimeparse.timeparse import timeparse

from ..base import BaseCommand
from ...overdue import get_overdue_items


class Command(BaseCommand):
    help_text = (
        "Count the number of overdue tasks by estimated time"
    )

    def __init__(self, *args, **kwargs):
        super(Command, self).__init__(*args, **kwargs)
        self._todoist_labels = None

    def add_command_line_options(self):
        super(Command, self).add_command_line_options()

    def execute(self, **kwargs):
        super(Command, self).execute(**kwargs)

        overdue_items = get_overdue_items(self.todoist_api)

        total_time = 0.0
        for item in overdue_items:
            total_time += self.get_hours_estimate(item)

        if kwargs['debug']:
            print '%d overdue items will take %.2f hours' % (
                len(overdue_items),
                total_time
            )
        else:
            worksheet = self.get_or_create_worksheet(
                'overdue time',
                ['date', 'n', 'time (hours)'],
            )
            worksheet.insert_row([
                datetime.datetime.now().date(),
                len(overdue_items),
                total_time,
            ], 2)

    def get_todoist_labels(self):
        if self._todoist_labels:
            return self._todoist_labels
        labels = self.todoist_api.labels.all()
        self._todoist_labels = {}
        for label in labels:
            self._todoist_labels[label['id']] = label['name']
        return self._todoist_labels

    def get_hours_estimate(self, item):
        labels = self.get_todoist_labels()
        hours = 0.0
        for label_id in item['labels']:
            label = labels[label_id]
            seconds_estimate = timeparse(label)
            if seconds_estimate is not None:
                hours += seconds_estimate / 60.0 / 60.0
        return hours



import datetime

from ..base import BaseCommand
from ...overdue import get_overdue_items


class Command(BaseCommand):
    help_text = (
        "Count the number of overdue tasks by priority"
    )

    def add_command_line_options(self):
        super(Command, self).add_command_line_options()

    def execute(self, **kwargs):
        super(Command, self).execute(**kwargs)

        overdue_items = get_overdue_items(self.todoist_api)

        counter = {'p1': 0, 'p2': 0, 'p3': 0, 'p4': 0}
        for item in overdue_items:
            priority = 'p' + str(4 - item['priority'] + 1)
            counter[priority] += 1

        if kwargs['debug']:
            for priority, count in sorted(counter.iteritems()):
                print '%4d overdue %s priority items' % (count, priority)
            print '-' * 30
            print '%4d overdue items TOTAL' % len(overdue_items)
        else:
            worksheet = self.get_or_create_worksheet(
                'overdue priority',
                ['date', 'n', 'p1', 'p2', 'p3', 'p4'],
            )
            worksheet.insert_row([
                datetime.datetime.now().date(),
                len(overdue_items),
                counter['p1'],
                counter['p2'],
                counter['p3'],
                counter['p4'],
            ], 2)



import argparse


# NOTE: this appears here so we can recursively use this functionality in
# recursive subcommands
def get_command_line_parser(command_line_parser, command_modules):
    """Public function for creating a parser to execute all of the commands
    in this sub-package.
    """
    subcommand_creator = command_line_parser.add_subparsers(
        title='SUBCOMMANDS',
    )
    for command_module in command_modules:
        command = command_module.Command(subcommand_creator)

        # this sets a default value for the command "option" so
        # that, when this Command is selected by argparse from the
        # command line, we know which common instance it
        # corresponds with. See run_subcommand function below.
        command.option_parser.set_defaults(command=command)
    return command_line_parser



import glob
import os
from setuptools import setup

# get all of the scripts
scripts = glob.glob(os.path.join("bin", "*"))

# read in the description from README
with open("README.md") as stream:
    long_description = stream.read()

github_url='https://github.com/deanmalmgren/todoist-tracker'

# read in the dependencies from the virtualenv requirements file
dependencies, dependency_links = [], []
filename = os.path.join("requirements", "python")
with open(filename, 'r') as stream:
    for line in stream:
        line = line.strip()
        if line.startswith("http"):
            dependency_links.append(line)
        else:
            package = line.split('#')[0]
            if package:
                dependencies.append(package)

# create the package
setup(
    name='todoist-tracker',
    version='0.1.2',
    description="track various metrics for todoist",
    long_description=long_description,
    url=github_url,
    download_url="%s/archives/master" % github_url,
    author='Dean Malmgren',
    author_email='dean.malmgren@datascopeanalytics.com',
    license='MIT',
    packages=[
        'todoist_tracker',
        'todoist_tracker.cli',
        'todoist_tracker.cli.overdue',
    ],
    scripts=scripts,
    install_requires=dependencies,
    dependency_links=dependency_links,
    zip_safe=False,
)



import argparse

from . import overdue
from .utils import get_command_line_parser

COMMAND_MODULES = (overdue, )


def get_cli_parser():
    command_line_parser = argparse.ArgumentParser(
        description="calculate and log various todoist statistics",
    )
    return get_command_line_parser(command_line_parser, COMMAND_MODULES)


def run_subcommand(args):
    """This function runs the command that is selected by this particular
    subcommand parser.
    """
    command = args.__dict__.pop("command")
    command.execute(**args.__dict__)
